from distutils.core import setup

setup(
    name="podium_api",
    packages=["podium_api", "podium_api.types"],
    version="0.0.21",
    description="API for Podium Motorsports Platform http://podium.live",
    author="Autosport Labs",
    author_email="sales@autosportlabs.com",
    url="https://github.com/autosportlabs/podium-api",
    download_url="https://github.com/autosportlabs/podium-api/archive/0.0.21.tar.gz",
    keywords=["motorsports", "telemetry", "live streaming"],
    classifiers=[],
)
